// Copyright 2018 the u-root Authors. All rights reserved
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// Package uio unifies commonly used io utilities for u-root.
//
// uio's most used feature is the Buffer/Lexer combination to parse binary data
// of arbitrary endianness into data structures.
package uio
